import Clutter from 'gi://Clutter';
import Meta from 'gi://Meta';
import { Extension } from 'resource:///org/gnome/shell/extensions/extension.js';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import Connections from './connections.js';
import { Keys } from './keys.js';
export default class WindowDesaturation extends Extension {
    connections;
    focused_window;
    settings;
    enable() {
        this.settings = this.getSettings();
        this.focused_window = global.display.focus_window;
        this.connections = new Connections();
        for (const window of this.windows()) {
            this.handle(window);
        }
        this.settings.connect('changed', () => this.refresh());
        const cursor = Meta.CursorTracker.get_for_display(global.display);
        this.connections.with(cursor, (it) => it.connect('position-invalidated', () => this.refresh()));
        this.connections.with(global.display, (it) => {
            it.connect('window-created', (_, window) => this.handle(window));
            it.connect('focus-window', (_, window) => this.update_focus(window));
        });
        this.connections.with(Main.overview, (it) => {
            it.connect('showing', () => this.refresh());
            it.connect('hidden', () => this.refresh());
        });
    }
    disable() {
        this.connections?.disconnect_all();
        this.connections = undefined;
        this.settings = undefined;
        for (const window of this.windows()) {
            this.clear(window);
        }
    }
    handle(window) {
        if (window.window_type !== Meta.WindowType.NORMAL)
            return;
        const actor = this.actor(window);
        if (!actor)
            return;
        const factor = this.factor(window);
        const effect = new Clutter.DesaturateEffect({ factor });
        actor.add_effect_with_name(this.uuid, effect);
    }
    clear(window) {
        const actor = this.actor(window);
        if (!actor)
            return;
        actor.remove_effect_by_name(this.uuid);
    }
    refresh() {
        for (const window of this.windows()) {
            this.update(window);
        }
    }
    update_focus(window) {
        if (this.focused_window === window)
            return;
        const before = this.focused_window;
        this.focused_window = window;
        this.update(before);
        this.update(window);
    }
    update(window) {
        const effect = this.effect(window);
        if (!effect)
            return;
        effect.factor = this.factor(window);
    }
    actor(window) {
        if (!window)
            return;
        return window.get_compositor_private();
    }
    effect(window) {
        if (!window)
            return;
        const actor = this.actor(window);
        if (!actor)
            return;
        const effect = actor.get_effect(this.uuid);
        if (!(effect instanceof Clutter.DesaturateEffect))
            return;
        return effect;
    }
    factor(window) {
        if (!window)
            return 0;
        if (Main.overview.visible && !this.handle_overview)
            return 0;
        if (window === this.focused_window)
            return this.foreground_factor;
        if (window.has_pointer() && this.handle_hover)
            return this.hover_factor;
        return this.background_factor;
    }
    windows() {
        return global.display.list_all_windows();
    }
    get background_factor() {
        // biome-ignore lint/style/noNonNullAssertion: this.settings is defined
        return this.settings.get_double(Keys.background_factor);
    }
    get foreground_factor() {
        // biome-ignore lint/style/noNonNullAssertion: this.settings is defined
        return this.settings.get_double(Keys.foreground_factor);
    }
    get hover_factor() {
        // biome-ignore lint/style/noNonNullAssertion: this.settings is defined
        return this.settings.get_double(Keys.hover_factor);
    }
    get handle_overview() {
        // biome-ignore lint/style/noNonNullAssertion: this.settings is defined
        return this.settings.get_boolean(Keys.desaturate_overview);
    }
    get handle_hover() {
        // biome-ignore lint/style/noNonNullAssertion: this.settings is defined
        return this.settings.get_boolean(Keys.desaturate_hover);
    }
}
